from __future__ import annotations
from enum import IntEnum, auto


class UnitProperty(IntEnum):
    LIFE_POINTS = 1
    MAX_LIFE_POINTS = auto()
    MOVE_POINTS = auto()
    MAX_MOVE_POINTS = auto()
    ACTION_POINTS = auto()
    MAX_ACTION_POINTS = auto()

    MELEE_ATTACK = auto()
    MELEE_DEFENSE = auto()
    MELEE_RANGE = auto()

    BOW_ATTACK = auto()
    BOW_DEFENSE = auto()
    BOW_RANGE = auto()

    MOUNT_ATTACK = auto()
    MOUNT_DEFENSE = auto()
    MOUNT_RANGE = auto()

    SIEGE_ATTACK = auto()
    SIEGE_DEFENSE = auto()
    SIEGE_RANGE = auto()

    @staticmethod
    def fromName(name: str) -> UnitProperty:
        if name not in unitProperty2Id:
            raise ValueError(f"Invalid unit property name {name}")
        return unitProperty2Id[name]

    def toName(self) -> str:
        return unitProperty2Name[self]


unitProperty2Name = {
    UnitProperty.LIFE_POINTS: "LIFE_POINTS",
    UnitProperty.MAX_LIFE_POINTS: "MAX_LIFE_POINTS",
    UnitProperty.MOVE_POINTS: "MOVE_POINTS",
    UnitProperty.MAX_MOVE_POINTS: "MAX_MOVE_POINTS",
    UnitProperty.ACTION_POINTS: "ACTION_POINTS",
    UnitProperty.MAX_ACTION_POINTS: "MAX_ACTION_POINTS",

    UnitProperty.MELEE_ATTACK: "MELEE_ATTACK",
    UnitProperty.MELEE_DEFENSE: "MELEE_DEFENSE",
    UnitProperty.MELEE_RANGE: "MELEE_RANGE",

    UnitProperty.BOW_ATTACK: "BOW_ATTACK",
    UnitProperty.BOW_DEFENSE: "BOW_DEFENSE",
    UnitProperty.BOW_RANGE: "BOW_RANGE",

    UnitProperty.MOUNT_ATTACK: "MOUNT_ATTACK",
    UnitProperty.MOUNT_DEFENSE: "MOUNT_DEFENSE",
    UnitProperty.MOUNT_RANGE: "MOUNT_RANGE",

    UnitProperty.SIEGE_ATTACK: "SIEGE_ATTACK",
    UnitProperty.SIEGE_DEFENSE: "SIEGE_DEFENSE",
    UnitProperty.SIEGE_RANGE: "SIEGE_RANGE",
}

unitProperty2Id = {
    "LIFE_POINTS": UnitProperty.LIFE_POINTS,
    "MAX_LIFE_POINTS": UnitProperty.MAX_LIFE_POINTS,
    "MOVE_POINTS": UnitProperty.MOVE_POINTS,
    "MAX_MOVE_POINTS": UnitProperty.MAX_MOVE_POINTS,
    "ACTION_POINTS": UnitProperty.ACTION_POINTS,
    "MAX_ACTION_POINTS": UnitProperty.MAX_ACTION_POINTS,

    "MELEE_ATTACK": UnitProperty.MELEE_ATTACK,
    "MELEE_DEFENSE": UnitProperty.MELEE_DEFENSE,
    "MELEE_RANGE": UnitProperty.MELEE_RANGE,

    "BOW_ATTACK": UnitProperty.BOW_ATTACK,
    "BOW_DEFENSE": UnitProperty.BOW_DEFENSE,
    "BOW_RANGE": UnitProperty.BOW_RANGE,

    "MOUNT_ATTACK": UnitProperty.MOUNT_ATTACK,
    "MOUNT_DEFENSE": UnitProperty.MOUNT_DEFENSE,
    "MOUNT_RANGE": UnitProperty.MOUNT_RANGE,

    "SIEGE_ATTACK": UnitProperty.SIEGE_ATTACK,
    "SIEGE_DEFENSE": UnitProperty.SIEGE_DEFENSE,
    "SIEGE_RANGE": UnitProperty.SIEGE_RANGE,
}

attackUnitProperties = {
    UnitProperty.MELEE_ATTACK: {
        "defense": UnitProperty.MELEE_DEFENSE,
        "range": UnitProperty.MELEE_RANGE,
    },
    UnitProperty.BOW_ATTACK: {
        "defense": UnitProperty.BOW_DEFENSE,
        "range": UnitProperty.BOW_RANGE,
    },
    UnitProperty.MOUNT_ATTACK: {
        "defense": UnitProperty.MOUNT_DEFENSE,
        "range": UnitProperty.MOUNT_RANGE,
    },
    UnitProperty.SIEGE_ATTACK: {
        "defense": UnitProperty.SIEGE_DEFENSE,
        "range": UnitProperty.SIEGE_RANGE,
    },
}

