from __future__ import annotations

from core.constants import CellValue, checkCellValue
from tools.tilecodes import mask4, code4
from .SetLayerCellCommand import SetLayerCellCommand
from core.logic.Logic import Logic


class SetObjectsCellCommand(SetLayerCellCommand):

    def check(self, logic: Logic) -> bool:
        value = self._value
        if not checkCellValue("objects", value):
            return False
        if self._unit is not None:
            return False
        cell = self._cell
        world = logic.world
        if not world.contains(cell):
            return False
        
        objectsValue = world.objects.getValue(cell)
        if value == CellValue.NONE:
            if objectsValue == CellValue.NONE:  # Value already removed
                return False
        else:
            if objectsValue != CellValue.NONE:  # Value already set
                return False
            if world.ground.getValue(cell) == CellValue.GROUND_SEA:  # Can't build on the sea
                return False
            
            # Impassable
            impassableLayer = world.impassable
            impassableValue = impassableLayer.getValue(cell)
            if impassableValue == CellValue.IMPASSABLE_RIVER:  # River case
                if value not in [CellValue.OBJECTS_ROAD_DIRT, CellValue.OBJECTS_ROAD_STONE]:  # Can only build a road
                    return False
                neighbors = impassableLayer.getNeighbors4(cell)
                mask = mask4(neighbors, CellValue.IMPASSABLE_RIVER)
                code = code4(mask)
                if code not in [6, 9]:  # Horizontal or vertical river
                    return False
            else:  # General case
                if impassableValue != CellValue.NONE:  # Can't build on mountains etc.
                    return False

        return True

    def execute(self, logic: Logic):
        cell = self._cell
        value = self._value
        world = logic.world
        world.objects.setValue(cell, value)
        world.objects.notifyCellChanged(cell)

        if self._fill:
            x, y = cell[0], cell[1]
            logic.addCommand(SetObjectsCellCommand((x + 1, y), self._value, self._unit, True))
            logic.addCommand(SetObjectsCellCommand((x - 1, y), self._value, self._unit, True))
            logic.addCommand(SetObjectsCellCommand((x, y + 1), self._value, self._unit, True))
            logic.addCommand(SetObjectsCellCommand((x, y - 1), self._value, self._unit, True))

