import logging

from core.constants import CellValue
from core.logic import Logic
from core.state import World, GameState
from tools.vector import vectorMulI, vectorDivI, vectorSubI
from ui import UserInterface, Theme
from ui.mode import EditGameMode, PlayGameMode

logging.basicConfig(level=logging.INFO, format='\r%(asctime)s %(filename)s:%(lineno)d: %(message)s')


# Load state
world = World((100, 80))
world.ground.fill(CellValue.GROUND_EARTH)
state = GameState(world)
logic = Logic(state)

# Create a user interface and run it
theme = Theme()
tileSize = theme.getTileset("ground").tileSize
theme.viewSize = vectorMulI((27, 15), tileSize)
userInterface = UserInterface(theme)
theme.init()
gameMode = EditGameMode(theme, logic)
userInterface.setGameMode(gameMode)
worldSize = vectorMulI(state.world.size, tileSize)
view = vectorDivI(vectorSubI(worldSize, theme.viewSize), (2, 2))
gameMode.viewChanged(view)
gameMode.notifyShowMessageRequested(
    "F3: load level.json<br>"
    "F5: save level.json<br>"
    "<br>"
    "Click to close this message", "elder")
userInterface.run()
userInterface.quit()

