from typing import Callable, NoReturn, Tuple, Optional

import pygame.draw
from pygame.surface import Surface

from ..Component import Component
from ...IUIEventHandler import IUIEventHandler
from ...Mouse import Mouse
from ...MouseWheel import MouseWheel
from ...theme.Theme import Theme


class Button(Component, IUIEventHandler):

    def __init__(self, theme: Theme,
                 surface: Surface,
                 action: Callable[[Tuple[bool, bool, bool]], NoReturn]):
        self.__surface = surface
        self.__action = action
        self.__highlight = False
        self.__mouseButtons: Optional[Tuple[bool, bool, bool]] = None
        super().__init__(theme, surface.get_size())

    def setSurface(self, surface: Surface):
        self.__surface = surface
        self.resize(surface.get_size())

    def render(self, surface: Surface):
        area = self.area
        if self.__mouseButtons is not None:
            area.move_ip((1, 1))
        if self.__highlight:
            color = self.theme.backgroundColor
            pygame.draw.rect(surface, color, area)
        surface.blit(self.__surface, area.topleft)

    # UI Event Handler

    def mouseEnter(self, mouse: Mouse) -> bool:
        self.__highlight = True
        self.__mouseButtons = None
        return True

    def mouseLeave(self) -> bool:
        self.__highlight = False
        self.__mouseButtons = None
        return True

    def mouseButtonDown(self, mouse: Mouse) -> bool:
        self.__mouseButtons = mouse.buttons
        return True

    def mouseButtonUp(self, mouse: Mouse) -> bool:
        if self.__mouseButtons is not None:
            self.__action(self.__mouseButtons)
            self.__mouseButtons = None
        return True

    def mouseWheel(self, mouse: Mouse, wheel: MouseWheel) -> bool:
        return True

    def mouseMove(self, mouse: Mouse) -> bool:
        return True
