from pygame.surface import Surface

from core.constants import CellValue
from core.state import World
from .LayerComponent import LayerComponent
from ...theme.Theme import Theme


class UnitsComponent(LayerComponent):
    def __init__(self, theme: Theme, world: World):
        super().__init__(theme, world.units, "units")
        self.__units = world.units

    def render(self, surface: Surface):
        super().render(surface)
        tileset = self.tileset.surface
        tilesRects = self.tileset.getTilesRects()

        renderer = self.createRenderer(surface)
        cellsSlice = renderer.cellsSlice
        cellMinX, cellMinY, _, _ = renderer.cellsBox
        cells = self.__units.cells[cellsSlice]

        valid = cells == CellValue.UNITS_UNIT
        for dest, value, cell in renderer.coords(valid):
            cellX, cellY = cellMinX + cell[0], cellMinY + cell[1]
            unit = self.__units.getUnit((cellX, cellY))
            if unit is not None:
                rects = tilesRects[unit.unitClass]
                surface.blit(tileset, dest, rects[unit.playerId])

